'****************************************************************
'*  Name    : Demo_ST7565R_SPI.BAS                              *
'*  Author  : Barak                                             *
'*  Notice  : Copyright (c) 2012                                *
'*          : All Rights Reserved                               *
'*  Date    : 5/15/2012                                         *
'*  Version : 1.0                                               *
'*  Notes   : demo GLCD with ST7565 controller and SPI          *
'*          : PIC18F46K22 is used for the demo. Any 18F with    *
'*          : MSSP module and enough memory/RAM can be used     * 
'*          : must Declare Stack_Size = 6 Or more               *
'*          : Not tested with 16F                               *
'****************************************************************
'
Device = 18F46K22
'
Config_Start
   FOSC = HSMP ; HS oscillator (medium power 4-16 MHz)
   PLLCFG = OFF ; Oscillator used directly
   PRICLKEN = OFF ; Primary clock can be disabled by software
   FCMEN = On ; Fail-Safe Clock Monitor enabled
   IESO = On ; Oscillator Switchover mode enabled
   PWRTEN = On ; Power up timer enabled
   BOREN = NOSLP ; Brown-out Reset enabled in hardware only and disabled in Sleep mode (SBOREN is disabled)
   BORV = 190 ; VBOR set to 1.90 V nominal
   WDTEN = SWON ; WDT is controlled by SWDTEN bit of the WDTCON register
   WDTPS = 128 ; 1:128
   CCP2MX = PORTC1 ; CCP2 input/output is multiplexed with RC1
   PBADEN = OFF ; PORTB<5:0> pins are configured as digital I/O on Reset
   CCP3MX = PORTE0 ; P3A/CCP3 input/output is mulitplexed with RE0
   HFOFST = On ; HFINTOSC output and ready status are not delayed by the oscillator stable status
   T3CMX = PORTC0 ; T3CKI is on RC0
   P2BMX = PORTC0 ; P2B is on RC0
   MCLRE = EXTMCLR ; MCLR pin enabled, RE3 input pin disabled
   STVREN = On ; Stack full/underflow will cause Reset
   LVP = OFF ; Single-Supply ICSP disabled
   XINST = OFF ; Instruction set extension and Indexed Addressing mode disabled (Legacy mode)
   Debug = OFF ; Disabled
   CP0 = OFF ; Block 0 (000800-001FFFh) code-protected
   CP1 = OFF ; Block 1 (002000-003FFFh) code-protected
   CP2 = OFF ; Block 2 (004000-005FFFh) code-protected
   CP3 = OFF ; Block 3 (006000-007FFFh) code-protected
   CPB = OFF ; Boot block (000000-0007FFh) code-protected
   CPD = OFF ; Data EEPROM not code-protected
   WRT0 = OFF ; Block 0 (000800-001FFFh) not write-protected
   WRT1 = OFF ; Block 1 (002000-003FFFh) not write-protected
   WRT2 = OFF ; Block 2 (004000-005FFFh) not write-protected
   WRT3 = OFF ; Block 3 (006000-007FFFh) not write-protected
   WRTC = OFF ; Configuration registers (300000-3000FFh) not write-protected
   WRTB = OFF ; Boot Block (000000-0007FFh) not write-protected
   WRTD = OFF ; Data EEPROM not write-protected
   EBTR0 = OFF ; Block 0 (000800-001FFFh) not protected from table reads executed in other blocks
   EBTR1 = OFF ; Block 1 (002000-003FFFh) not protected from table reads executed in other blocks
   EBTR2 = OFF ; Block 2 (004000-005FFFh) not protected from table reads executed in other blocks
   EBTR3 = OFF ; Block 3 (006000-007FFFh) not protected from table reads executed in other blocks
   EBTRB = OFF ; Boot Block (000000-0007FFh) not protected from table reads executed in other blocks
Config_End
'
Optimiser_Level = 3
Xtal = 16                           ' Declare Fosc 
Declare Stack_Size = 8              ' only need 6 for GLCD    
'---[OSC config]---------------------        
    OSCCON  = %01110000             ' IRCF = 111 = internal osc 16MHz; SCS = 00 Primary OSC from CONFIG1H
    OSCCON2 = %00000100             ' PRISD OSC drive circuit on
    OSCTUNE = %00000000             ' TUN = 000000 = Osc module runs at factory calibration
'---[Ports Config]-------------------       
    ANSELA  = %00000000               
    ANSELB  = %00000000                     
    ANSELC  = %00000000
    ANSELD  = %00000000
    ANSELE  = %00000000                     
    TRISA   = %00000000              
    TRISB   = %01111111             ' PORTB all inputs but leave PORTB.7 as output for SW UART TX      
    TRISC   = %01010000             ' Set RC.4 Input For MISO, RC.6 TX1, all other outputs          
    TRISD   = %00000000              
	TRISE   = %00000000              
'-----[SPI config]-------------------
    SSP1CON1 = %00000001            ' configure SPI Master, FOSC/16 so SCL is 1MHz 
    Symbol SSP1EN  = SSP1CON1.5     ' SSP Enable bit
    Symbol CKP1    = SSP1CON1.4     ' Clock Polarity Select
    Symbol SMP1    = SSP1STAT.7     ' Data input sample bit
    Symbol CKE1    = SSP1STAT.6     ' Clock Edge Select bit
    Symbol BF1     = SSP1STAT.0     ' buffer full status bit 1 = buffer full
    Symbol SSP1IF  = PIR1.3         ' interrupt flag - last bit set   
'_____________________________________________________
'    
    Include "GLCD_ST7565R_SPI.inc"  ' ST7565R/SPI functions - must be at the begining ! 
'____________________________________________________    
'
'---[Demo Variables]---------------------------------    
    Dim i As Byte                                    
    Dim j As Byte                                   
    Dim temp_word As SWord                          
    Dim Strtext As String * 24                      
'                           
Main:  
    DelayMS 250                                     
    '---intilaize the SPI in Mode 0------------------
    ' note: demo using SPI#1 of 18FXXK22, remove 
    ' the 1 from the symbols for single SPI PICs
    '------------------------------------------------
    SMP1   = 0                                      ' Data input sampled at middle of data output time
    CKE1   = 1                                      ' CPHA - transmit on falling edge active to idle 
    CKP1   = 0                                      ' CPOL - idle state is low level
    SSP1EN = 1                                      ' Enable SPI  
    '---intilaize the LCD----------------------------
    GoSub InitEADOGM1286                            ' intialize the LCD      
    DelayMS 10
    GoSub GlcdWriteCmd[0xA5]                        ' all pointson- black scrren 
    DelayMS 500                                      
    GoSub GlcdWriteCmd[0xA4]                        ' normal display
    Cls                                             ' clear all the junk from the memory
    (*--fonts Notes----------------------------------
    not compatible with Pprint but supporrts 
    Fontconverter (for PDS) table structure.
    Large or special fonts tables are processed
    by separate sub and selected by the macro:
    Glcd_SetFont(TableName,ASCII-OFFSET,Invert)
    TableName is FontConverter's header table
    the table must have the hight in pixels in the 
    first element. Each font table must have the 
    font's number of columns in the first element  
    ASCII-OFFSET is 32 for a full table _0 to _95
    for modified tables i.e., only numbers
    use other offset values. See example below
    Invert: 1=invert the font(negative), 0=normal
    The internal font Font.inc has different table 
    structure. For speed and simplicity it is handled 
    by a separte sub and selected by the macro:
    Glcd_IntFont(Invert)
    Invert: 1=invert the Font(negative), 0=normal 
    '----------------------------------------------*)
    Glcd_IntFont(1)                                 ' macro to use internal font 1=inverted 
    Print At 3,15, "ST7565R SPI GLCD"                                      
    GoSub DrawRectangle[1,14,23,97,9]               ' add one pixel around the balck background [1=set pixel,x0,y0,width,hight]
    DelayMS 2000
    Cls
    Glcd_SetFont(Verda_B__14o25,0x30,0)             ' select font, offset = 48, not inverted, we only have ASCII 0-9 in Verda_B__14o25 table so the offset is 48
    Print At 0,70, "99"                             ' same char print ok - becuse we keep a copy of WREG
    Glcd_SetFont(ARIAL_BOLD11,0x20,1)               ' select font, offset = 32, inverted
    Print At 0,30, "XYZ"
    Glcd_IntFont(0)                                 ' back to standard font not inverted
    Print At 7,0, "PDS GLCD"
    '
    '---use PDS GLCD lib----------------------------
    ' commands Plot, UnPlot, Line, circle, box and 
    ' LCDWrite arguments as per PDS manual 
    '-----------------------------------------------
    j = 48
    For i = 20 To 100
        Plot j, i                                   ' plot a line          
    Next
    DelayMS 1000
    For i = 50 To 70
        UnPlot j, i                                 ' unplot part of the line
        DelayMS 10
    Next
    Line 1,10,10,102,48                             ' draw line
    Line 1,10,10,10,55                              ' draw line
    Circle 1,60,33,15                               ' draw a Circle
    Box 1, 100,38,10                                ' draw a box
    LCDWrite 6,10,  [0xFF,0x7F,0x3F,0x1F,_          ' draw a shape using LCDWrite
                    0x0F,0x07,0x03,0x01]
    LCDWrite 5,10,  [0xFF,0xFE,0xFC,0xF8,_
                    0xF0,0xE0,0xC0,0x80]
    '---other GLCD functions-------------------------
    ' Other functions that may be usefull
    ' arguments are explained in the comments
    '------------------------------------------------
    GoSub FillRectangle[1,71,49,39,13]              ' draw a filled rectangle [set_clear,x0,y0,width,hight]
    Line 0,98,49,68,62                              ' run a Clear diagonal line in the rectangle
    GoSub DrawRectangle[1,13,23,20,10]              ' draw line rectangle (not a square) [set_clear,x0,y0,width,hight]
    DelayMS 5000
    '---print without storage in image buffer--------
    ' can be used to show text and then refresh the 
    ' display to original image. we don't use Print 
    ' becuase it always updates the memory
    '------------------------------------------------
    GoSub BlankDisplay                              ' clear the display but not the memory buffer!
    Strtext = "OFF THE RECORD!"                     ' some string
    Cursor 3,8
    i = 0                                           
    Repeat                                          ' print the string directly - don't use Print! 
        j = Strtext[i]
        If j = 0 Then Break                         ' exit when we get the null
        GoSub OffRecordPrint[j]                     ' prints a charchter without storing in buffer
        Inc i   
    Until i = 20 
    DelayMS 5000
    GoSub GlcdBuffDump                              ' restore the display before the off record print
    DelayMS 5000
    '---demo scrolling numbers counter---------------
    temp_word = 9999
    Glcd_SetFont(Verda_B__14o25,0x30,0)             ' load a large font, table only has numbers so offset is 48
    j = 0
    Cls 
    While 1 = 1
        Inc temp_word
        Print At 3,30,SDec5 temp_word
        DelayMS 100
        j = (j + 1) & MAXLIN                        ' numbers 0 to max line resolution (i.e., 63)   
        GoSub GlcdScroll[j]                         ' scroll up (invert value to scroll down)
        DelayMS 100
    Wend
'
'---include tables for all the fonts that will be used
' at the end of the code        
'______________________________________________________
    Include "Font.inc"
    Include "ARIAL_BOLD_11.inc"
    Include "Verda_Bold_14_nums.inc"   
'______________________________________________________ 
'    
