'****************************************************************
'*  Name    : LCD_ST7565R_SPI.inc                               *
'*  Author  : Barak                                             *
'*  Notice  : Copyright (c) 2012 All Rights Reserved            *
'*  Date    : 05/15/2012                                        *
'*  Version : V.1.0                                             *
'*  Notes   : Use GLCD with ST7565R controller via HW SPI       *
'*          : the key PDS GLCD lib commands are supported       *
'*          : by redirecting to subs in this file               *
'*          : Developed with EA DOGM128-6 in 3.3V Mode          *
'*          : It is not possible to read the LCD RAM via SPI    *
'*          : Therefore, the code includes 1024 elements array  *
'*          : That mirrors the LCD RAM.                         *
'*          :                                                   *
'****************************************************************


'---SPI/GLCD Defines]------------------
    Symbol CS1B     = PORTC.7                       ' CS1B is the EADOGM128-6 "Chip Select Not" pin  
    High CS1B                                       ' make sure it does't get any    
    Symbol A0       = PORTC.2                       ' LCD A0: 0 = Command, 1 = data              
'---[constants for EA DOGM128-6 128x64]-------------                           
    Symbol MAXCOL    = 127                          ' max column number
    Symbol XRES      = 128                          ' total number of columns                           
    Symbol MAXLIN    =  63                          ' max line number 
    Symbol MAXPAG    =   7                          ' max page number
   
'---[GLCD Variables used in ASM code]---------------    
    Dim strbyte         As Byte                     ' byte for Print redirect
    Dim radius          As Byte                     ' the radius of a circle
    Dim xpos            As Byte                     ' column position 0 - 127
    Dim ypos            As Byte                     ' page position 0 - 7
    Dim lpos            As Byte                     ' line position 0 - 63 for scroling and effects
    Dim x0              As Byte                     ' x start point
    Dim y0              As Byte                     ' y start point
    Dim x1              As Byte                     ' x end point
    Dim y1              As Byte                     ' y end point
    Dim ascii_os        As Byte                     ' the ASCII offset for a font table
    Dim pix_byte        As Byte                     ' pixel black or white
    Dim pix_bit         As Bit                      ' pixel bit flag black or white
    Dim invert_f        As Bit = 0                  ' flag to invert text
    Dim font_flag       As Bit = 0                  ' pointer to font table
    Dim fontpages       As Byte                     ' font's number of pages/lines 
    Dim lcd_tempword    As Word                     ' general purpose
    Dim fontaddress     As Word                     ' address of font table
    Dim wTblPtr As TBLPTRL.Word                     ' Alias SFR As word variables
'
'===[GLCD SETUP}=====================================    
  Declare LCD_Type graphic
'  
    '---[disable PDS GLCD lib]-----------------------     
    #disable Print,_
    GLCD_CLS,_
    GLCD_READ,_
    GLCD_READTEXT,_
    GLCD_WRITE,_
    GLCD_PIXEL,_
    GLCD_PLOT,_
    GLCD_CURSOR,_
    GLCD_CIRCLE,_
    GLCD_BOX,_
    GLCD_LINE,_
    GLCD_LOW_LEVEL
    '================================================
    GoTo EndOfGLCD                                  ' jump over all functions
    '================================================
GlcdLibRedirect:
    '---[Redirect PDS GLCD lib]----------------------
    Asm 
    '------------------------------------------------
    GLCD@CLS                                        ' Cls - clear the screen and the memory image
    GoTo ClsRedirect                  
    Return
    '------------------------------------------------
    GLCD@CUR                                        ' Cursor ypos, xpos - Position the cursor on graphic LCD
    Movwf xpos                                      ' WREG = XPOS
    Movff GEN, ypos                                 ' GEN = YPOS (Line)            
    GoTo CursorRedirect                  
    Return
    '-----------------------------------------------
    Print                                           ' Print or Print at ypos, xpos 
    Movwf strbyte                                   ' WREG = character to display
    GoTo PrintRedirect            
    Return
    '------------------------------------------------
    __LCD_PLOT_                                     ' Plot ypos, xpos
    Movwf xpos                                      ' WREG = XPOS
    Movff GEN, lpos                                 ' GEN = lpos is the row number 0-63 (Not Page/Line!)            
    Bsf pix_bit                                     ' set bit plot a pixel
    GoTo PlotRedirect                  
    Return
    '-----------------------------------------------
    __LCD_UNPLOT_                                   ' UnPlot ypos, xpos
    Movwf xpos                                      ' WREG = XPOS
    Movff GEN, lpos                                 ' GEN = lpos is the row number 0-63 (Not Page/Line!)            
    Bcf pix_bit                                     ' clear bit to unplot
    GoTo PlotRedirect                  
    Return
    '-----------------------------------------------
    __LCD_LINE_                                     ' Line set_clear, x-start, y-start, x-end, y-end
    Movwf y1                                        ' WREG = YPOS end
    Movff GENH, x1                                  ' GENH = XPOS end
    Movff GEN2H, lpos                               ' GEN2H = YPOS start
    Movff GEN2, xpos                                ' GENH = XPOS start
    Bsf pix_bit                                     ' set bit plot a pixel
    Btfss BPF,2                                     ' test BPF Bit 2 - skip next if bit is 1
    Bcf pix_bit                                     ' clear bit to unplot
    GoTo LineRedirect
    Return
    '-----------------------------------------------
    __LCD_CIRCLE_                                   ' Circle set_clear, xpos, ypos, radius
    Movwf radius                                    ' WREG = radius
    Movff GEN2, x0                                  ' GEN2 = YPOS
    Movff GEN2H, y0                                 ' GEN2H = XPOS 
    Bsf pix_bit                                     ' set bit plot a pixel
    Btfss BPF,2                                     ' test BPF Bit 2 - skip next if bit is 1
    Bcf pix_bit                                     ' clear bit to unplot
    GoTo CircleRedirect
    Return
    '------------------------------------------------
    __LCD_BOX_                                      ' box set_clear, xpos, ypos, size (only a square - see Rectangle below)
    Movwf radius                                    ' WREG = radius
    Movff GEN2, x0                                  ' GEN2 = YPOS
    Movff GEN2H, y0                                 ' GEN2H = XPOS 
    Movff BPF, pix_byte                             ' BPF to a var bit#2 is the plot/unplot flag
    GoTo BoxRedirect
    Return
    '------------------------------------------------
    __LCD_WRITEBYTE_                                ' LCDWrite ypos, xpos,[byte,{byte etc...}]
    Movwf Char                                      ' the byte to write at page
    Movff GEN, ypos                                 ' load to page var
    Movff GENH, xpos                                ' load to col var
    Incf GENH                                       ' col +1 for next then byte if any
    GoTo LCDWriteRedirect
    Return
    '-----------------------------------------------
    EndAsm
'
' fake use of GLCD Library commands to intialize PDS 
' variables. Otherwise, comment any redirect that will   
' Not be used from the ASM in GlcdLibRedirect
InitGlcdLib:
    Plot 0, 0
    UnPlot 0, 0
    Line 0, 0, 0, 2, 0
    Circle 0, 0, 0, 2
    Box 0, 0, 0, 2
    LCDWrite 0, 0, [0x00]
    Cls
    Return
'
'---[Macro Selects FontConverter font]--------------- 
' hacked from similar macro from ILI9320.inc by Les
' Input     : TBLPTRL\H hold the font table address
'           : the second parameter is the Font table offset
'             typically 32 but can be other value for custom tables
'           : the third parameter is inverting the font 1 = inverted = negative
' Output    : fontaddress is the start address of the font table
'           : fontpages is the number of pages needed for the font
'
$define Glcd_SetFont(FontName,OffSet,FontInvert)    '           
    wTblPtr = VarPtr FontName                       '
    ascii_os = OffSet                               ' 
    invert_f = FontInvert                           '
    font_flag =  1                                  '
    _SetFont
'
_SetFont Macro-
    GoSub __SetFont
    Endm
'
#ifMacro- _SetFont
__SetFont:
    Tblrd*+                                         ' get the font pixel height from the first element in the font table
    lcd_tempword = TABLAT                           ' load the font hight in pixels
    Tblrd*+                                         ' Dummy read to increment the code memory address
    fontaddress = wTblPtr                           ' Store the address of the font table
    fontpages = lcd_tempword / 8                    ' calc the number of pages
    PRODL = lcd_tempword // 8                       ' get the remainder
    If PRODL <> 0 Then                              ' any remainder we need another page
        Inc fontpages                               ' increment the number of pages
    EndIf
    Return
#endIfMacro-
'
'---[return to the internal font]--------------------
$define Glcd_IntFont(FontInvert)    '   
    font_flag =  0                  '    
    invert_f = FontInvert           ' 
'
'---[Other GLCD Variables]---------------------------
' placed after the ASM so ASM code in first bank    
    Dim Char            As Byte                     ' data byte to LCD
    Dim cmdbyte         As Byte                     ' byte for commands, can't be the same as used to redirect
    Dim lcd_temp        As Byte                     ' Temp byte for LCD functions
    Dim lcd_indx        As Byte                     ' Index Byte For LCD functions                       
    Dim buff_byte       As Byte                     ' byte from LCD memory image
    Dim x_glcd          As Byte                     ' for glcd x calc
    Dim y_glcd          As Byte                     ' For glcd y calc     
    Dim xadd            As SByte                    ' inc or dec x 
    Dim yadd            As SByte                    ' inc or dec y
    Dim xdelta          As Byte                     ' delta = abs(x1-x0)
    Dim ydelta          As Byte                     ' delta = abs(y1-y0)
    Dim SPI_BYTE_IN     As Byte                     ' SPI input - not applicable to the LCD
    Dim fontcolumns     As Byte                     ' fornt number of columns
    Dim font_indx       As Byte                     ' index for FontConverter cdata tables
    Dim xyerr           As SWord                    ' xy error for Bresenham algorithm
    Dim charaddress     As Word                     ' index for font Cdata tables
    Dim glcdbuff[1024]  As Byte = 0                 ' array to mirror the 128x8 memory of the GLCD becuae we can't read it
'
'---[SPI Function]-----------------------------------
SPI_GLOBAL:                                         ' generic SPI sub             
    Pop SSP1BUF                                     ' trasmit a byte from stack
    While SSP1IF = 0 : Wend                         ' wait for end of transaction
    SSP1IF = 0                                      ' clear the SSP flag
    SPI_BYTE_IN = SSP1BUF                           ' nothing to do with the LCD use to read from other slaves
    Return  
'
'----------------------------------------------------                                                       
PrintFontConverter:                                 ' handles FontConverter tables structure                              
    High A0                                         ' send as Data
    Low CS1B                                        ' CSN
    '------------------------------------------------
    lcd_temp = (strbyte - ascii_os) * 2             ' (ascii value - offset) * 2 becuase address is a word value
    charaddress  = CRead fontaddress + lcd_temp     ' the address for first element
    fontcolumns  = CRead charaddress                ' the number of columns of the Font are in the first element  
    '---auto next line-------------------------------
    If xpos + fontcolumns > MAXCOL Then             ' only for the 6x8 font need to use var to make it adjustable
        xpos = 0                                    ' first column
        ypos = ypos + fontpages                     ' next page/line
        If ypos > 7 Then ypos = 0                   ' over-type text in first page - uncomment to cancel over-type
        GoSub CursorRedirect                        ' cursor to page/column coordinates
    EndIf 
    '------------------------------------------------
    x0 = xpos                                       ' keep a copy of original x
    y0 = ypos                                       ' keep a copy of original y
    For y_glcd = 0 To fontpages - 1                 ' go through each page
        font_indx = y_glcd + 1                      ' move one element from the start for each page starting element number 1 (0 has the num of columns)
        xpos = x0                                   ' start at intial col                                   
        ypos = y0 + y_glcd                          ' calc the page 
        GoSub CursorRedirect                        ' go to col/page coordinates
        x_glcd = 0                                  ' run all the columns for this line
        Repeat
            Char = CRead charaddress + font_indx    ' get a bitmap byte
            If invert_f = 1 Then Char = ~Char          ' invert if selected
            GoSub GlcdWriteChar                     ' write to LCD
            font_indx = font_indx + fontpages       ' jump to the next col value in the curent page
            Inc xpos                                ' keep track of the xpos for the memory image buffer
            Inc x_glcd                              ' inc the columns counter
        Until x_glcd = fontcolumns                  ' until the number of col per page
    Next
    ypos = y0                                       ' back to intial page/line
    High CS1B
    Return      
'---END PDS Font system-----------------------------
' faster print dedicated only to standard Font.inc
'---[writes from Print cmd to LCD]------------------
PrintRedirect:                                      ' write the string from a PDS Print                              
    If font_flag = 0 Then                           ' use the internal font deom PDS Font.inc     
        High A0                                     ' send as Data
        '---auto next line---------------------------
        If xpos + 6 > MAXCOL Then                   ' only for the 6x8 font need to use var to make it adjustable
            xpos = 0                                ' first column
            ypos = (ypos + 1) & 0x07                ' next page/line
            GoSub CursorRedirect                    ' cursor to page/column coordinates
        EndIf 
        '--------------------------------------------
        Low CS1B                                    ' CSN
        For lcd_indx = 0 To 5                       ' for the built-in 6x8 pixel font
            charaddress = strbyte * 6 + lcd_indx
            lcd_temp = CRead8 Font[charaddress]
            If invert_f = 1 Then lcd_temp = ~lcd_temp
            Char = lcd_temp
            GoSub GlcdWriteChar
            Inc xpos                                ' for buffer pointer - increase the column address after each byte
        Next
        High CS1B
    Else                                            ' its a lrge font go to special sub so we can do standard font fast
        GoSub PrintFontConverter   
    EndIf
    @Movf strbyte, W                                '!!! must put the byte back to WREG in case we print the same char again!!!
    Return    
'
'---[writes data byte to LCD]------------------------
GlcdWriteChar:     
    High A0                                         ' send as Data                  
    glcdbuff[ypos * XRES + xpos] = Char             ' update the memory bufer 
    Low CS1B                                        ' Select EADOG LCD as slave
    GoSub SPI_GLOBAL[Char]                          ' push char to SSBUF
    High CS1B
    Return
'        
'---[writes command byte to LCD]---------------------
GlcdWriteCmd:                               
    Pop cmdbyte
    Low A0                                          ' send as a command
    DelayUS 10
    Low CS1B                                        ' Select EADOG LCD as slave
    GoSub SPI_GLOBAL[cmdbyte]                       ' push cmdbyte to SSBUF
    High CS1B
    Return   
'    
'---[init EADOG-M1286-SPI @ 3.3V ]-------------------    
InitEADOGM1286:
    For lcd_indx = 0 To 13
        lcd_temp = CRead EADOGM12863 + lcd_indx     ' load intialiazation command from Cdata table
        GoSub GlcdWriteCmd[lcd_temp]                ' send the command
    Next
    Return
'
'---[cursor to page/column coordinate]---------------
CursorRedirect:                                     ' needs ypose and xpos if accesed directly                                          
    lcd_temp = (ypos & 0x0F) | 0xB0                 ' page address + Page 0 (0xB0)
    GoSub GlcdWriteCmd[lcd_temp]
    lcd_temp = (xpos >> 4)| 0x10                    ' column address high nibble + %00010000 
    GoSub GlcdWriteCmd[lcd_temp]                     
    lcd_temp = xpos & 0x0F                          ' column address low nibble
    GoSub GlcdWriteCmd[lcd_temp]
    Return
'
LCDWriteRedirect:                                   ' redirects the LCDWrite x,y,[Var2,{Var2 etc...}]
    GoSub CursorRedirect  
    GoSub GlcdWriteChar
    Return
'
'---[goes to any line 0-63]--------------------------
GlcdScroll:                                        ' normally lines are moving up to scroll down invert the value
    Pop lpos                                       ' line value 0 - MAXLIN
    lcd_temp = (lpos & 0x3F) | 0x40                ' line position + %01000000 
    GoSub GlcdWriteCmd[lcd_temp]
    Return    
'
GlcdWriteBuffer:                                    ' write without updating the memory buffer     
    Pop Char 
    High A0                                         ' send as Data
    Low CS1B                                        ' Select EADOG LCD as slave
    GoSub SPI_GLOBAL[Char]                          ' push the LCD Byte to the stack so it goes to SSBUF
    High CS1B
    Return
'
GlcdBuffDump:                                       ' for graphics - dump the entire memory image to the display
    High A0                                         ' send as Data
    Low CS1B
    ypos = 0
    Repeat                                          ' go through the enite memory buffer
        xpos = 0
        Repeat
            GoSub CursorRedirect
            buff_byte = glcdbuff[ypos * XRES + xpos]
            GoSub GlcdWriteBuffer[buff_byte]
            Inc xpos        
        Until xpos > MAXCOL    
        Inc ypos
    Until ypos > MAXPAG 
    High CS1B
    Return
'
GlcdClearPage:                                      ' clears a page without updating the memory buffer     
    High A0                                         ' send as Data
    Low CS1B
    For x_glcd = 0 To MAXCOL                                       
        GoSub SPI_GLOBAL[0x00]                      ' push the LCD Byte to the stack so it goes to SSBUF     
    Next
    High CS1B
    Return
'
BlankDisplay:                                       ' clears the display but keeps the memory image for refresh
    For y_glcd = 0 To MAXPAG                        ' go through the pages                
        Cursor y_glcd,0
        GoSub GlcdClearPage
    Next
    Return
'
'---[for PDS Cls]-----------------------------------------
ClsRedirect:                                        ' clears the display and the memory image
    For y_glcd = 0 To MAXPAG                        ' go through the pages                
        Cursor y_glcd,0                             ' cursor to start of page
        GoSub GlcdClearPage
    Next
    Clear glcdbuff
    Return
'
'---[set/clear pixel]-------------------------------
PlotRedirect:                                       ' add the pixel to the memeory image byte - i.e., fake read-modify-write
    ypos = lpos/8                                   ' get the page                               
    GoSub CursorRedirect                            ' cursor to page/column coordinates 
    Char = glcdbuff[ypos * XRES + xpos]             ' "read" - the byte from memeory image 
    lcd_temp = 0x01 << lpos//8                      ' set only the bit corresponding with the row number
    If pix_bit = 1 Then                             ' "modify"    
        Char = Char | lcd_temp                      ' set the pixel in the memeory byte  
    Else    
        Char = Char & ~lcd_temp                     ' clear the pixel in the memeory byte = buff_byte & ~lcd_temp            ' clear the pixel in the memeory byte 
    EndIf
    GoSub GlcdWriteChar                             ' "write"
    Return'

'---[Bresenham's algorithm thanks Wikipedia]--------    
LineRedirect:
    xdelta = Abs(x1 - xpos)                         ' delta x
    ydelta = Abs(y1 - lpos)                         ' delta y
    If xpos < x1 Then 
        xadd = 1                                    ' to increase x   
    Else
        xadd = -1                                   ' to decrease x
    EndIf
    If lpos < y1 Then
        yadd = 1                                    ' to increase y            
    Else
        yadd = -1                                   ' to decrease y
    EndIf
    xyerr = xdelta - ydelta                         ' intial pixel error
    While 1 = 1
        GoSub PlotRedirect 
        If xdelta > ydelta Then
            If xpos = x1 Then Break                 ' there are more x exit when x is complete
        Else
            If lpos = y1 Then Break                 ' there are more y exit when y is complete
        EndIf
        If xyerr * 2 > -ydelta Then
            xyerr = xyerr - ydelta
            xpos = xpos + xadd                      ' x is moving        
        EndIf    
        If xyerr * 2 < xdelta Then
            xyerr = xyerr + xdelta
            lpos = lpos + yadd                      ' y is moving   
        EndIf    
    Wend 
    Return    


'---[midpoint circle algorithm thanks Wikipedia]----    
Circle4Points:                                      ' set a pixel at each quadrant
    xpos = x0 + x_glcd
    lpos = y0 + y_glcd
    GoSub PlotRedirect
    If x_glcd != 0 Then 
        xpos = x0 - x_glcd
        lpos = y0 + y_glcd
        GoSub PlotRedirect
    EndIf
    If y_glcd != 0 Then 
        xpos = x0 + x_glcd
        lpos = y0 - y_glcd
        GoSub PlotRedirect 
    EndIf
    If x_glcd != 0 And y_glcd != 0 Then 
        xpos = x0 - x_glcd
        lpos = y0 - y_glcd
        GoSub PlotRedirect 
    EndIf
    Return

CircleRedirect:                                     ' MidPoint Circle algorithm from Wikipedia
    xyerr = -radius                                 ' error
    x_glcd = radius
    y_glcd = 0
    While x_glcd >= y_glcd
        GoSub Circle4Points
        If x_glcd != y_glcd Then
            Swap x_glcd,y_glcd
            GoSub Circle4Points
            Swap x_glcd,y_glcd
        EndIf
        xyerr = xyerr + y_glcd
        y_glcd = y_glcd + 1
        xyerr = xyerr + y_glcd
        If xyerr >= 0 Then
            xyerr = xyerr - x_glcd
            x_glcd = x_glcd - 1
            xyerr = xyerr - x_glcd
        EndIf
    Wend
    Return     
    
DrawRectangle:
    Pop y1                                          ' hight 
    Pop x1                                          ' width
    Pop y0                                          ' y start point
    Pop x0                                          ' x start point
    Pop pix_byte                                    ' 1 = set or 0 = clear pixel 
    pix_bit = pix_byte.0                            ' can't pass a bit argument
    For xpos = x0 To x0 + x1
        lpos = y0
        GoSub PlotRedirect
        lpos = y0 + y1
        GoSub PlotRedirect    
    Next
    For lpos = y0 To y0 + y1
        xpos = x0
        GoSub PlotRedirect
        xpos = x0 + x1
        GoSub PlotRedirect  
    Next
    Return

BoxRedirect:
    pix_byte = pix_byte >> 2                         ' shift from bit#2 for compatibility with DrawRectangle sub
    GoSub DrawRectangle[pix_byte,x0,y0,radius,radius]' use the rectangle sub to draw a square 
    Return

'---[subs for bitmap ops via the memory buffer]------
'---[set/clear pixel in memory image]----------------
BuffSetPixel:                                       ' add the pixel to the memeory image byte - i.e., fake read-modify-write
    Pop xpos                                        ' the column
    Pop lpos                                        ' the row number 0-63 (not page/line!)
    ypos = lpos/8                                   ' get the page                               
    lcd_tempword = ypos * XRES + xpos 
    buff_byte = glcdbuff[lcd_tempword]              ' "read" - the byte from memeory image 
    lcd_temp = 0x01 << lpos//8                      ' set only the bit corresponding with the row number
    If pix_byte = 1 Then                            ' "modify"
        buff_byte = buff_byte | lcd_temp            ' set the pixel in the memeory byte 
    Else
        buff_byte = buff_byte & ~lcd_temp           ' clear the pixel in the memeory byte 
    EndIf
    glcdbuff[lcd_tempword] = buff_byte              ' Write back to ememry image
    Return

'---R-M-W to memory image then dump to lcd----------
FillRectangle:                                      
    Pop y1                                          ' hight 
    Pop x1                                          ' width
    Pop y0                                          ' y start point
    Pop x0                                          ' x start point
    Pop pix_byte                                    ' 1 = set or 0 = clear pixel 
    For x_glcd = x0 To x0 + x1
        For y_glcd = y0 To y0 + y1
            GoSub BuffSetPixel[y_glcd,x_glcd]
        Next
    Next
    GoSub GlcdBuffDump
    Return

OffRecordPrint:                                     ' print a char without putting in memory image
    pop strbyte
    For lcd_indx = 0 To 5                           ' for the built-in 6x8 pixel font
        charaddress = strbyte * 6 + lcd_indx
        lcd_temp = CRead8 Font[charaddress]
        GoSub GlcdWriteBuffer[lcd_temp]
    Next
    Return
'==============CDATA END OF CODE=====================    
'---[EADOGM162/3.3V/SPI initialization]--------------    
EADOGM12863:                              
    CData As Byte   0x40,_                          ' START LINE 0 
                    0xA1,_                          ' ADC reversed
                    0xC0,_                          ' normal COM0~COM63 0xC8 reverses everything from the bottom
                    0xA6,_                          ' Display normal
                    0xA2,_                          ' LCD bias set 1/9 (duty 1/65)
                    0x2F,_                          ' Booster, regulator, Follower On 
                    0xF8,_
                    0x00,_                          ' Internal booster ratio x4
                    0x27,_                          ' 0x27
                    0x81,_                          ' 0x81
                    0x16,_                          ' 0x16 Contrast set
                    0xAC,_
                    0x00,_                          ' Static indicator not set
                    0xAF                            ' Display On    

'====================================================
EndOfGLCD:
'====================================================
